from typing import Any


path = []
temp_path = ''


def parse_params(url: str) -> list[str]:
    params = []
    temp_path = ''
    for i in url:
        if (i == '/' and len(temp_path) > 0):
            path.append(temp_path)
            temp_path = ''
            
        if (i == ':') or len(temp_path) > 0:
            temp_path += i
    else:
        path.append(temp_path) if temp_path else None
    return params

def insert_params(path: str, params: dict[str, Any]) -> str:
    for k, v in params.items():
        path = path.replace(k, v)
    return path


import argparse
import asyncio

from loyalcat.http import HttpClient, Route


p = argparse.ArgumentParser()
p.add_argument('-a', '--action', help='動作', default='ci')
p.add_argument('-c', '--commit_hash', help='VCSのコミットハッシュ', default=None)
p.add_argument('-p', '--project_id', help='対象のプロジェクトID', default=None)
p.add_argument('-s', '--status', help='結果', default=None)
args = p.parse_args()


async def create_pipeline(api: HttpClient):
    if args.commit_hash is None or args.project_id is None:
        raise Exception('-c, -pのどちらかが不足しています')
    res = await api.request(Route('POST', '/api/pipelines/:id', params={':id': args.project_id}), body={'commitHash': args.commit_hash})
    print(res.get('url'))
    while True:
        status_check = await api.request(Route('GET', '/api/pipelines/:id', params={':id': res.get('id')}))
        if status_check.get('isDone', False) is True:
            await api.session.close()
            print(status_check)
            if status_check.get('type', 'success') == 'success':
                print('success')
                exit(0)
            else:
                raise Exception('CI failure')
        await asyncio.sleep(3)

async def finished_pipeline(api: HttpClient):
    if args.commit_hash is None or args.status is None:
        raise Exception('-c, -sが不足しています')
    await api.request(Route('POST', '/api/pipelines/:id/update', {':id': args.commit_hash}), body={'status': args.status})

async def _main():
    api = HttpClient().create_session()

    match args.action:
        case 'ci': await create_pipeline(api)
        case 'finish': await finished_pipeline(api)
    await api.session.close()
def main() -> None:
    asyncio.run(_main())
        
    


from setuptools import setup



packages = ['loyalcat.types']

setup(
    name='loyalcat',
    version='0.0.1',
    install_requires=['aiohttp'],
    author='yupix',
    author_email='yupi0982@outlook.jp',
    license='MIT',
    python_requires='>=3.10, <4.0',
    packages=packages,
    entry_points={'console_scripts': ['loyalcat = loyalcat.cli:main']},
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python :: 3.10',
        'Natural Language :: Japanese',
        'License :: OSI Approved :: MIT License',
    ],
)




import json
from typing import Any, Generic, Literal, NewType, Self, TypeVar, TypeVarTuple
import aiohttp

from loyalcat.types.endpoints import ENDPOINTS
from loyalcat.utils import insert_params

R = TypeVar('R')


class Route:
    def __init__(
        self,
        method: Literal['GET', 'POST'],
        path: ENDPOINTS,
        params: dict[str, Any] | None = None,
    ):
        self.path: str = insert_params(path, params) if params else path
        self.method: str = method


async def json_or_text(response: aiohttp.ClientResponse):
    text = await response.text(encoding='utf-8')

    try:
        if 'application/json' in response.headers['Content-Type']:
            return json.loads(text)
    except KeyError:
        pass


class HttpClient:
    def __init__(self) -> None:
        self.session: aiohttp.ClientSession
        self.url = 'https://loyalcat.akarinext.org'

    def create_session(self) -> Self:
        self.session = aiohttp.ClientSession(
            headers={'Content-type': 'application/json'},
        )
        return self

    async def request(self, route: Route, body: dict[str, Any] | None = None) -> R:
        async with self.session.request(
            route.method, self.url + route.path, json=body
        ) as res:
            data = await json_or_text(res)
            if 300 > res.status >= 200:
                return data  # type: ignore
            else:
                raise Exception(data, res.status)

